;; Time.Lsp - optional file for writing actual times to c2012_data.dta           
;; CADALYST Systems Benchmark 2012 v5.4 (released Jul-2011)
;; (c)Copyright 1996-2011, Art Liddle and CADALYST Magazine
;;
;; NOTE:
;; AutoCAD 2012 update: Disabled Viewport Controls
;;                      Adjusted 3D/2D tests to balance times
;;                      Added Battery Rundown Test (99 Loops)
;;                      Renamed files  
;;
;; 6-01-2010 (C2011, v5.3b): disabled navigation cube; renamed files for ACAD 2011; 
;; 8-01-09: Used Autodesk's Gstest.arx files for updated 3D tests.
;; 7-01-08: Revised 3D tests: removed Flat, and Gouraud; added Conceptual and Realistic
;;                            adjusted number of repeats; adjusted base times

;; NOTE:
;; 2D test supports R2000 and newer
;;         works with any driver
;; New 3D test requires AutoCAD 2009 (Update #3) or newer.


(setq DWG1 "c2012_1")
(setq DWG2 "c2012_2")
(setq DWG3 "c2012_3")
(setq DWG4 "c2012_4")
(setq DWG5 "c2012_5")
(setq DWG6 "c2012_6")
(setq DWG7 "c2012_7")
(setq DWG8 "c2012_8")

(setq DATA_FILE "c2012_data.dta")
(setq CNFG_FILE "c2012_cnfg.dta")
(setq TRNSFR_FILE "c2012_next")
(setq TRNSFR_FILE_EXT "c2012_next.scr")
(setq TIME_FILE "c2012_time")
(setq 2DTMP_FILE "c2012_2d.tmp")
(setq 3DTMP_FILE "c2012_3d.tmp")

(setq 3DCUR_FILE "c2012_current.dta")
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(setq 3DBASE_FILE base_file)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;



(setq LOOP_FILE "c2012_loop")
(setq LOOP_FILE_EXT "c2012_loop.scr")
(setq NUM_2D 4)
;;;;<<---- number of 2d test dwgs
(setq NUM_3D 4)
;;;;<<---- number of 3d test dwgs

;; Error handler
(defun *ERROR* (MSG)
  (if (/= MSG "function cancelled")
    (if	(= MSG "quit / exit abort")
      (princ)
      (princ (strcat "\nError-->  " MSG "\n"))
    ) ; end second if
    (princ)
  ) ; end first if
)   ; end defun *error*

;; Create a list of lists (pairs) from a file
;; Single data per line in file
(defun LIST_IN (A_FILE / A_LIST ITEM1 ITEM2)
  (setq A_LIST (list))
  (while (setq ITEM1 (read-line A_FILE))
    (setq ITEM2 (read-line A_FILE))
    (setq A_LIST (append A_LIST (list (list ITEM1 ITEM2))))
  ) ; end while
)   ; end defun list_in

;; Strip out numbers from list of pairs 
(defun STRIP (A_LIST / B_LIST CNTR ITEM)
  (setq B_LIST (list))
  (setq CNTR 1)
  ;; change to 2 to omit first pair
  (while (setq ITEM (nth (1- CNTR) A_LIST))
    (setq B_LIST (append B_LIST (list (atof (cadr ITEM)))))
    (setq CNTR (1+ CNTR))
  ) ; end while
  (setq B_LIST B_LIST)
)   ; end defun strip


;; Read current test results in from file
(defun OUTPUT_3D ()
  (setq FNAME 3DCUR_FILE)
  (setq FILE (open FNAME "r"))
  (setq 3D_LIST_ALL (LIST_IN FILE))
  (close FILE)

  ;;strip away notes, leaving just numbers
  (setq TMP_LIST 3D_LIST_ALL)
  ;; for later use
  (setq 3D_LIST_ALL (STRIP 3D_LIST_ALL))
  (setq LOOPS (fix (car 3D_LIST_ALL)))
  (setq 3D_LIST_ALL (cdr 3D_LIST_ALL))

  ;;get name of tests
  (setq LEN (length TMP_LIST))
  (setq NUM_TEST (/ LEN NUM_3D LOOPS))
  (setq TST_NAME (list))
  (setq CNTR 1)
  ;; skip number of loops
  (repeat NUM_TEST
    (setq NAM (list (car (nth CNTR TMP_LIST))))
    (setq TST_NAME (append TST_NAME NAM))
    (setq CNTR (1+ CNTR))
  ) ;end repeat

  ;;figure average times
  (setq LEN (length 3D_LIST_ALL))
  (setq OFFSET (fix (/ LEN LOOPS)))
  (setq CNTR 0)
  (setq 3D_CUR (list))
  (repeat OFFSET
    (setq SUM 0)
    (setq CNTRX 0)
    (repeat LOOPS
      (setq SUM (+ SUM (nth (+ CNTR (* CNTRX OFFSET)) 3D_LIST_ALL)))
      (setq CNTRX (1+ CNTRX))
    ) ;end repeat loops
    (if	(< SUM 0.05)
      (setq SUM 0.05)
    )
    (setq AVG (list (/ SUM LOOPS)))
    (setq 3D_CUR (append 3D_CUR AVG))
    (setq CNTR (1+ CNTR))
  ) ;end repeat offset

  ;; Read base 3dtest results in from file
  (setq FNAME 3DBASE_FILE)
  (setq FILE (open FNAME "r"))
  (setq 3D_BASE (LIST_IN FILE))
  (close FILE)

  ;;remove first pair- description
  (setq 3D_BASE (cdr 3D_BASE))

  ;;strip away notes, leaving just numbers
  (setq 3D_BASE (STRIP 3D_BASE))

  ;;calculate ratios of current times to base times
;;;(setq 3d_ratios (mapcar '/ 3d_base 3d_cur))
  (setq 3D_RATIOS 3D_CUR)

  ;; remove the setup/initial render times
  (setq B_LIST (list))
  (setq C_LIST (list))
  (setq REPX (1- NUM_TEST))
  (setq CNTR 1)
  ;; change to 2 to omit first pair
  (while (setq ITEM (nth (1- CNTR) 3D_RATIOS))
    (setq B_LIST (append B_LIST (list ITEM)))
    (repeat REPX
      (setq ITEM (nth CNTR 3D_RATIOS))
      (setq C_LIST (append C_LIST (list ITEM)))
      (setq CNTR (1+ CNTR))
    ) ;end repeat
    (setq CNTR (1+ CNTR))
  ) ; end while
  (setq 3D_RATIOS_NEW C_LIST)
;;;;  (setq 3d_setup b_list)
  (setq NUM_TEST (1- NUM_TEST))
  ;; adjust for removing setup ratios

  ;;calcualte the total 3d average time
  (setq LEN (length 3D_RATIOS))
  (setq REP (fix LEN))
  (setq CNTR 0)
  (setq SUM NIL)
  (setq TOTAL 0.0)
  (repeat REP
    (setq SUM (nth CNTR 3D_RATIOS))
    (setq CNTR (1+ CNTR))
    (setq TOTAL (+ TOTAL SUM))
  ) ;end repeat
  (setq 3D_INDX (/ TOTAL LEN))
;;;(setq 3d_indx total)

  ;;calculate the test 3d average ratios; include setup numbers
  (setq LEN (length 3D_RATIOS))
  (setq REP (fix (/ LEN NUM_3D)))
  (setq CNTR 0)
  (setq X NIL)
  (setq 3D_TEST (list))
  (setq ROW_LIST (list))
  (repeat REP
    (setq SUM 0)
    (setq CNTRX 0)
    (repeat NUM_3D
      (setq SUM (+ SUM (setq X (nth (+ CNTR (* CNTRX REP)) 3D_RATIOS))))
      (setq CNTRX (1+ CNTRX))
      (setq ROW_LIST (append ROW_LIST (list X)))
    ) ;end repeat loops
    (setq AVG (list (/ SUM NUM_3D)))
;;;  (setq avg (list sum))
    (setq 3D_TEST (append 3D_TEST AVG))
    (setq CNTR (1+ CNTR))
  ) ;end repeat offset

  ;;calcualte the drawing 3d average 
  (setq LEN (length 3D_RATIOS))
  (setq REP (fix (/ LEN NUM_3D)))
  (setq CNTR 0)
  (setq 3D_DWGS (list))
  (repeat NUM_3D
    (setq SUM 0)
    (repeat REP
      (setq SUM (+ SUM (nth CNTR 3D_RATIOS)))
      (setq CNTR (1+ CNTR))
    ) ;end repeat loops
    (setq AVG (list (/ SUM REP)))
    ;;  (setq avg (list sum))
    (setq 3D_DWGS (append 3D_DWGS AVG))
  ) ;end repeat offset

  ;; Begin writng index information to file and drawing
  (setq FNAME DATA_FILE)
  (setq FILE (open FNAME "a"))

(setq dwg5 "1st Dwg")
(setq dwg6 "2nd Dwg")
(setq dwg7 "3rd Dwg")
(setq dwg8 "4th Dwg")

  (write-line
    (strcat
      "Drawing===>>                "
      DWG5
      "   "
      DWG6
      "   "
      DWG7
      "   "
      DWG8
      "     Test"
    )
    FILE
  ) ; end write-line

  (write-line
    "                                                                      (avg)"
    FILE
  ) ; end write-line
  (write-line
    "- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -"
    FILE
  )
  (setq NUM_TEST (1+ NUM_TEST))
  ;; adjust for output of ALL test numbers
  (setq CNTR 0)
  (setq CNTRX 0)
  (repeat NUM_TEST
    (setq XX (nth CNTR ROW_LIST))
    (setq X (cond
	      ((< XX 10) "      ");;;;;;;;;;;;;;;;;;;;
	      ((and (>= XX 10) (< XX 100)) "     ");;;;;;;;;;;;;;;;
	      ((and (>= XX 100) (< XX 10000)) "    ");;;;;;;;;;;;;;;;;;  deleted two spaces
	    ) ;end cond
    ) ;end setq
    (repeat NUM_3D
      (setq X (strcat X
		      (rtos (setq XX (nth CNTR ROW_LIST)) 2 1)
		      (cond
			((< XX 10) "       ")
			((and (>= XX 10) (< XX 100)) "      ")
			((and (>= XX 100) (< XX 10000)) "     ")
		      ) ;end cond
	      ) ;end strcat
      ) ;end setq
      (setq CNTR (1+ CNTR))
    ) ;end repeat num_3d
    (setq X (strcat (nth CNTRX TST_NAME) X))
    (setq X (strcat X " " (rtos (nth CNTRX 3D_TEST) 2 1)))
    (write-line X FILE)
    (setq CNTRX (1+ CNTRX))
    (setq X "")
  ) ;end repeat num_test

  ;;write dwg averages
  (write-line
    "- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -"
    FILE
  )

  (setq CNTR -1)
  (write-line
    (strcat                        ;;"
      "Average Test Time By DWG =    "
      (rtos (nth (setq CNTR (1+ CNTR)) 3D_DWGS) 2 1)
      "      "

      (rtos (nth (setq CNTR (1+ CNTR)) 3D_DWGS) 2 1)
      "      "
      (rtos (nth (setq CNTR (1+ CNTR)) 3D_DWGS) 2 1)
      "      "

      (rtos (nth (setq CNTR (1+ CNTR)) 3D_DWGS) 2 1)
    )
    FILE
  )

    ;write total index number
  (write-line                            ;;"
    (strcat "Average Test Time =           " (rtos 3D_INDX 2 1))
    FILE
  )
  (write-line "*****************************" FILE)

  (write-line "" FILE)
  (close FILE)
)   ;end output_3d



(setq FNAME DATA_FILE)
(setq FILE (open FNAME "a"))
(write-line "******** Test Data:  Actual Times (sec)" FILE)
(close FILE)


;; Check if 3d tests were run
(setq FNAME 3DCUR_FILE)
(setq FILE (open FNAME "r"))
(setq 3D_LIST_ALL (LIST_IN FILE))
(close FILE)
(setq LEN (length 3D_LIST_ALL))
;;output 3d results if yes
(if (> LEN 2)
  (OUTPUT_3D)
)
